@@ -1,4 +1,4 @@
-{-# LANGUAGE OrPatterns, GADTs, LambdaCase, NamedFieldPuns #-}
+{-# LANGUAGE OrPatterns, GADTs, LambdaCase, NamedFieldPuns, TemplateHaskellQuotes #-}
 module GHC.Debugger.Runtime where
 
 import Data.IORef
@@ -6,8 +6,8 @@ import Control.Monad.Reader
 import qualified Data.List as L
 
 import GHC
+import GHC.Utils.Outputable
 import GHC.Types.FieldLabel
-import GHC.Tc.Utils.TcType
 import GHC.Runtime.Eval
 import GHC.Runtime.Heap.Inspect
 
@@ -29,17 +29,11 @@ obtainTerm key = do
     -- cache miss: reconstruct, then store.
     Nothing ->
       let
-        -- For boring types we want to get the value as it is (by traversing it to
-        -- the end), rather than stopping short and returning a suspension (e.g.
-        -- for the string tail), because boring types are printed whole rather than
-        -- being represented by an expandable structure.
-        depth i = if isBoringTy (GHC.idType i) then maxBound else defaultDepth
-
         -- Recursively get terms until we hit the desired key.
         getTerm = \case
-          FromId i -> GHC.obtainTermFromId (depth i) False{-don't force-} i
+          FromId i -> GHC.obtainTermFromId defaultDepth False{-don't force-} i
           FromPath k pf -> do
-            term <- getTerm k
+            term <- obtainTerm k
             liftIO $ expandTerm hsc_env $ case term of
               Term{dc=Right dc, subTerms} -> case pf of
                 PositionalIndex ix -> subTerms !! (ix-1)
@@ -49,7 +43,12 @@ obtainTerm key = do
                     Nothing -> error "Couldn't find labeled field in dataConFieldLabels"
               NewtypeWrap{wrapped_term} ->
                 wrapped_term -- regardless of PathFragment
-              _ -> error "Unexpected term for the given TermKey"
+              RefWrap{wrapped_term} ->
+                wrapped_term -- regardless of PathFragment
+              _ -> error ("Unexpected term for the given TermKey because <term> should have been expanded before and we're getting a path fragment!\n" ++ showPprUnsafe (ppr key <+> ppr k <+> ppr pf))
+          FromCustomTerm _key _name ctm -> do
+            -- For custom terms return them straightaway.
+            liftIO $ expandTerm hsc_env ctm
        in do
         term <- getTerm key
         liftIO $ modifyIORef tc_ref (insertTermCache key term)
@@ -65,18 +64,12 @@ obtainTerm key = do
 expandTerm :: HscEnv -> Term -> IO Term
 expandTerm hsc_env term = case term of
   Term{val, ty} -> cvObtainTerm hsc_env defaultDepth False ty val
-  (NewtypeWrap{}; RefWrap{}) -> do
-    -- TODO: we don't do anything clever here yet
-    return term
-  -- For other terms there's no point in trying to expand
-  (Suspension{}; Prim{}) -> return term
-
--- | A boring type is one for which we don't care about the structure and would
--- rather see "whole" when being inspected. Strings and literals are a good
--- example, because it's more useful to see the string value than it is to see
--- a linked list of characters where each has to be forced individually.
-isBoringTy :: Type -> Bool
-isBoringTy t = isDoubleTy t || isFloatTy t || isIntTy t || isWordTy t || isStringTy t
-                || isIntegerTy t || isNaturalTy t || isCharTy t
-
+  RefWrap{wrapped_term} -> do
+    wt' <- expandTerm hsc_env wrapped_term
+    return term{wrapped_term=wt'}
+  NewtypeWrap{wrapped_term} -> do
+    wt' <- expandTerm hsc_env wrapped_term
+    return term{wrapped_term=wt'}
+  Suspension{val, ty} -> cvObtainTerm hsc_env defaultDepth False ty val
+  Prim{} -> return term
 
